package com.qihoo.speechrecognition.ui;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.ShortBuffer;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import com.qihoo.speechrecognition.QihooSpeechRecognizer;
import com.qihoo.speechrecognition.QihooSpeechConfig;
import com.qihoo.speechrecognition.R;



import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.speech.RecognitionListener;
import android.speech.SpeechRecognizer;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewTreeObserver;
import android.view.ViewTreeObserver.OnGlobalLayoutListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class ASRDialog extends Dialog {

	private static final String TAG = "ASRDialog";
	public static final String KEY_SPEECH_CONTENT = "speech_content";
	
	public static final String KEY_APP_ID="appid";
	public static final String KEY_SECURET_KEY="securetKey";
	public static final String KEY_STYLE="style";
	
	public static final int INITSTATE_UNINIT=-1;
	public static final int INITSTATE_SUCCESS=0;
	public static final int INITSTATE_APPID_ERROR=1;
	public static final int INITSTATE_SECUREKEY_ERROR=2;
	public static final int INITSTATE_RECOGNIZER_ERROR=3;
	public static final int INITSTATE_PARAMS_NULL=4;
	
	
	
    private Context mContext;
    private Collection mListeners = null;
    private int mInitstate = INITSTATE_UNINIT;
    
    private static final int EVENT_RESULT=1;
    private static final int EVENT_PART_RESULT=2;
	
	private QihooSpeechRecognizer mSpeechRecognizer = null;
	private QihooSpeechConfig     mSpeechConfig = null;
	
	private String mAppid="";
	private String mSecuretKey="";
	
	private TextView txtTitle;
	private ImageView imgNetworkDisconnect;
	private ImageView imgRecognition;
	private ImageView imgSpeechMicrophoneBg;
	private ImageView imgAmplitude;
	private ImageView imgMicrophone;
	private Button btnCancel;
	private Button btnStop;
	private Button btnRetry;
	/**
	 * imgAmplitude
	 */
	private int heightAmplitudeView;
    
    private ASRDialog(Context context) {
        super(context);
        // TODO Auto-generated constructor stub
        this.mContext = context;
    }
    public ASRDialog(Context context,int style, Bundle params){
    	super(context,style);
    	this.mContext = context;
    	mInitstate = INITSTATE_UNINIT;
    	if(null == params){
    		mInitstate = INITSTATE_PARAMS_NULL;
    		return;
    	}
    	mAppid = params.getString(KEY_APP_ID);
    	mSecuretKey = params.getString(KEY_SECURET_KEY);
    	if(null == mAppid || TextUtils.equals(mAppid, "")){
    		mInitstate = INITSTATE_APPID_ERROR;
    		return;
    	}
    	if(null == mSecuretKey || TextUtils.equals(mSecuretKey, "")){
    		mInitstate = INITSTATE_SECUREKEY_ERROR;
    		return;
    	}
    	mInitstate = INITSTATE_SUCCESS;
        
    }
 
    public int getInitState(){
    	return mInitstate;
    }
    
    
    public void setListener(ASRDialogListener listener){
    	if(null == mListeners){
    		mListeners = new HashSet();
    	}
    	mListeners.add(listener);
    }
    
    
    public String getSDKVersion(){
    	if (null == mSpeechRecognizer) {
    		return null;
    	} 
    	return mSpeechRecognizer.getSDKVersion();
    }
    
    /*
     * when call onCreate? 
     *  show() or construct() ?
     * */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        // TODO Auto-generated method stub
        super.onCreate(savedInstanceState);
        if(UIConstants.UI_DEBUG) Log.i(TAG,"onCreate()");
        setContentView(R.layout.speech_recognition);
        findView();
		if(!initSpeechRecognizer()){
			mInitstate = INITSTATE_RECOGNIZER_ERROR;
			return;
		}
		startSpeechListening();
		//setOnDismissListener(null);
    }
    
    /**
	 * 
	 */
	protected void onDestroy() {
		if(UIConstants.UI_DEBUG) Log.i(TAG, "onDestroy");
		if (null != mSpeechRecognizer) {
			mSpeechRecognizer.destroy();
		}
	}
	
	private void findView() {
		txtTitle = (TextView)findViewById(R.id.txt_title);
		imgNetworkDisconnect = (ImageView)findViewById(R.id.img_network_disconnect);
		imgRecognition = (ImageView)findViewById(R.id.img_recognition);
		imgSpeechMicrophoneBg = (ImageView)findViewById(R.id.img_speech_microphone_bg);
		imgAmplitude = (ImageView)findViewById(R.id.img_amplitude);
		imgMicrophone = (ImageView)findViewById(R.id.img_microphone);
		btnCancel = (Button)findViewById(R.id.btn_cancel);
		btnStop = (Button)findViewById(R.id.btn_stop);
		btnRetry = (Button)findViewById(R.id.btn_retry);
		
		btnCancel.setOnClickListener(onClickListener);
		btnStop.setOnClickListener(onClickListener);
		btnRetry.setOnClickListener(onClickListener);
		
		final ViewTreeObserver observer = imgAmplitude.getViewTreeObserver();
		observer.addOnGlobalLayoutListener(new OnGlobalLayoutListener() {
			@Override
			public void onGlobalLayout() {
				Log.i(TAG, "onGlobalLayout");
				imgAmplitude.getViewTreeObserver().removeGlobalOnLayoutListener(this);
				heightAmplitudeView = imgAmplitude.getHeight();
				// 初始值 30
				setViewHeight(imgAmplitude, (int)(heightAmplitudeView*0.3f));
			}
		});
	}
	
	/**
	 * 
	 */
	private View.OnClickListener onClickListener = new View.OnClickListener() {
		
		@Override
		public void onClick(View v) {
			int id = v.getId();
			if(R.id.btn_cancel == id){
				mSpeechRecognizer.cancel();
				onDestroy();
				dismiss();
			}else if(R.id.btn_stop == id){
				speechStop();
			}else if(R.id.btn_retry == id){
				speechRetry();
			}
		}
	};

	/**
	 * 
	 */
	private void speechStop() {
		if(UIConstants.UI_DEBUG) Log.i(TAG,"speechStop()");
		/*txtTitle.setText(R.string.title_speech_recognitioning);
		imgRecognition.setVisibility(View.VISIBLE);
		btnStop.setVisibility(View.GONE);
		*/
		
		mSpeechRecognizer.stopListening();
	}
	
	/**
	 * 
	 */
	private void speechRetry() {
		txtTitle.setText(R.string.title_speech_ask_question);
		imgNetworkDisconnect.setVisibility(View.GONE);
		imgMicrophone.setVisibility(View.VISIBLE);
		imgAmplitude.setVisibility(View.VISIBLE);
		imgSpeechMicrophoneBg.setVisibility(View.VISIBLE);
		btnStop.setVisibility(View.VISIBLE);
		btnRetry.setVisibility(View.GONE);
		
		startSpeechListening();
	}
	
	/**
	 * 
	 */
	private boolean initSpeechRecognizer() {
		mSpeechRecognizer = QihooSpeechRecognizer.createSpeechRecognizer(mContext, mAppid, mSecuretKey);
		if(null == mSpeechRecognizer){
			return false;
		}
		mSpeechRecognizer.setRecognitionListener(mListener);
		mSpeechConfig = new QihooSpeechConfig();
		mSpeechConfig.setmBUseEndPoint(false);
		mSpeechConfig.setmConnTimeout(3);           //wifi
		mSpeechConfig.setmReadTimeout(4);           //wifi
		mSpeechConfig.setmNonWifiConnTimeout(5);    //nowifi
		mSpeechConfig.setmNonWifiReadTimeout(10);   //nowifi 
		mSpeechRecognizer.setConfig(mSpeechConfig);
		return true;
	}
	
	/**
	 * 
	 */
	private void startSpeechListening() {
		mSpeechRecognizer.cancel();
		mSpeechRecognizer.startListening();
	}
	
	private RecognitionListener mListener = new RecognitionListener() {

		/*
		 * 
		 */
		@Override
		public void onResults(Bundle results) {
			if(UIConstants.UI_DEBUG) Log.i(TAG, "onResults(results= " + ((null == results) ? "null" : results.toString()) + ")");
			if (null == results) {
				onDestroy();
				dismiss();
				return;
			}
			
			notifyListeners(EVENT_RESULT,results);
			
			onDestroy();
			dismiss();
		}

		/*
		 *
		 */
		@Override
		public void onError(int error) {
			if( UIConstants.UI_DEBUG) Log.i(TAG, "onError(err=" + error + ")");
			Log.d(TAG, "Error return from service, No. = " + error);
			
			//
			imgRecognition.setVisibility(View.GONE);
			imgSpeechMicrophoneBg.setVisibility(View.GONE);
			imgAmplitude.setVisibility(View.GONE);
			imgMicrophone.setVisibility(View.GONE);
			imgNetworkDisconnect.setVisibility(View.VISIBLE);
			
			
			String str = "";
			switch (error) {
			case SpeechRecognizer.ERROR_AUDIO:
				str = "录音机出错,查看录音权限";
				break;

			case SpeechRecognizer.ERROR_SERVER:
				str = "服务器出错";
				break;

			case SpeechRecognizer.ERROR_NETWORK:
				str = "网络错误";
				break;

			case SpeechRecognizer.ERROR_NETWORK_TIMEOUT:
				str = "网络不给力";
				break;

			case SpeechRecognizer.ERROR_NO_MATCH:
				str = "没有识别结果";
				break;

			case SpeechRecognizer.ERROR_CLIENT:
				str = "客户端错误";
				break;

			case SpeechRecognizer.ERROR_SPEECH_TIMEOUT:
				str = "没有检测到语音";
				break;

			case SpeechRecognizer.ERROR_RECOGNIZER_BUSY:
				str = "识别器较忙，稍后再试";
				break;
			default:
				str = "其他错误";
				break;
			}
			
			//txtTitle.setText(R.string.title_speech_network_disconnect);
			txtTitle.setText(str);
			btnStop.setVisibility(View.GONE);
			btnRetry.setVisibility(View.VISIBLE);
		}

		@Override
		public void onReadyForSpeech(Bundle params) {
			if(UIConstants.UI_DEBUG) Log.i(TAG, "onReadyForSpeech()");
		}

		/*
		 * 
		 */
		@Override
		public void onEndOfSpeech() {
			if(UIConstants.UI_DEBUG) Log.i(TAG, "onEndOfSpeech()");
			txtTitle.setText(R.string.title_speech_recognitioning);
			imgRecognition.setVisibility(View.VISIBLE);
			btnStop.setVisibility(View.GONE);
		}

		@Override
		public void onPartialResults(Bundle partialResults) {
			if(UIConstants.UI_DEBUG) Log.i(TAG, "onPartialResults()");
			if(null == partialResults){
				return;
			}
			/*
			ArrayList<String> strList = partialResults.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION);
			if (null != strList && strList.size() > 0 && !TextUtils.equals(strList.get(0), "")) {
				String result = strList.get(0);
				LogUtils.d(TAG, "recognition result = " + result);
				
				//txtTitle.setText(result);
				notifyListeners(EVENT_PART_RESULT,partialResults);
				//intent.putExtra(KEY_SPEECH_CONTENT, result);
				//setResult(RESULT_OK, intent);
			} //else {
			 //	LogUtils.d(TAG, "no recognition result");
			//}
			*/
			notifyListeners(EVENT_PART_RESULT,partialResults);
		}

		@Override
		public void onEvent(int eventType, Bundle params) {
			if(UIConstants.UI_DEBUG) Log.i(TAG, "onEvent()");
		}

		@Override
		public void onBufferReceived(byte[] buffer) {
			//Log.i(TAG, "onBufferReceived");
			if (buffer == null || buffer.length == 0) {
				Log.i(TAG, "buffer empty");
				return;
			}
			//Log.i(TAG, "get audio data length =" + buffer.length);

			int value = 0;
			int scale = 10;
			if (buffer == null || buffer.length == 0) {
				value = 0;
			} else {
				// 
				ShortBuffer sb = ByteBuffer.wrap(buffer, 0, buffer.length).order(ByteOrder.LITTLE_ENDIAN).asShortBuffer();

				int len = sb.remaining();
				short[] sr = new short[len];
				sb.get(sr, 0, len);

				for (int i = 0; i < sr.length; i++) {
					if (sr[i] > 0) {
						value = value + sr[i];
					} else {
						value = value - sr[i];
					}
				}

				value = value / (sr.length + 1);
				value = Math.max(value, 0); //
				value = Math.min(value, 100 * scale); //
				value = value / scale; //
				
//				Log.i(TAG, "value=" + value);
//				Log.i(TAG, "imgAmplitude height" + heightAmplitudeView);
//				Log.i(TAG, "now height" + heightAmplitudeView*(value/100.0f));
				
				//
				float height40p = heightAmplitudeView * ((70-30) / 100.0f);
				float height30p = heightAmplitudeView * 0.3f;
				float finalHeight = height30p + height40p*(value/100.0f);
				setViewHeight(imgAmplitude, (int)finalHeight);
			}
		}

		@Override
		public void onBeginningOfSpeech() {
			if(UIConstants.UI_DEBUG) Log.i(TAG, "onBeginningOfSpeech()");
		}

		@Override
		public void onRmsChanged(float rmsdB) {
			if(UIConstants.UI_DEBUG) Log.i(TAG, "onRmsChanged()");
		}
	};
	
	/**
	 * 
	 * @param view
	 * @param height
	 */
	private void setViewHeight(View view, int height) {
		RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams)view.getLayoutParams();
		params.height = height;
		view.setLayoutParams(params);
	}
	
	private void notifyListeners(int event, Bundle result) {
		
        Iterator iter = mListeners.iterator();
        while (iter.hasNext()) {
            ASRDialogListener listener = (ASRDialogListener) iter.next();
            if(EVENT_RESULT == event){
            	listener.onResults(result);
            }else if(EVENT_PART_RESULT == event){
            	listener.onPartialResults(result);
            }
        }
    }
	
	/*
	 *  点击回退按钮
	 * */
	@Override  
	public boolean onKeyDown(int keyCode, KeyEvent event) {  
		if(UIConstants.UI_DEBUG) Log.i(TAG,"onKeyDown(keyCode= " + keyCode + ")");
	    if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0) {
	    	onDestroy();
	    	dismiss();
	        return true;  
	    }  
	      
	    return super.onKeyDown(keyCode, event);  
	}  
	
	/*
	 *  点击Dialog后面的activity 
	 *  add this method here  when touch back activity  current dialog not close
	 * */
	
	public boolean onTouchEvent (MotionEvent event){
		if(UIConstants.UI_DEBUG) Log.i(TAG, "onTouchEvent(event=" + event + ")");
		return true;
	}
	
}
